@@ -171,37 +171,54 @@ class AccidentalsEngraver(Engraver):
                 im.get_pixbuf().render_to_drawable(widget.window, gc,
                         0, 0,
                         self.m_xpos + x,
-                        accidental_y_offset[acc] + staff_yoffset
+                        int(accidental_y_offset[acc] + staff_yoffset
                          + dimentions[self.m_fontsize].linespacing*y/2
-                         + accidental_y_offset[acc],
+                         + accidental_y_offset[acc]),
                         -1, -1, gtk.gdk.RGB_DITHER_NONE, 0, 0)
                 x += dimentions[self.m_fontsize].accidental_widths[acc] + 1
 
 
 class KeySignatureEngraver(Engraver):
-    def __init__(self, timepos, fontsize, key, clef):
+    def __init__(self, timepos, fontsize, old_key, key, clef):
         Engraver.__init__(self, timepos, fontsize)
+        self.m_old_key = old_key
         self.m_key = key
         self.m_clef = clef
+        self.m_old_accidentals = mpdutils.key_to_accidentals(old_key)
         self.m_accidentals = mpdutils.key_to_accidentals(key)
     def set_xpos(self, dict):
         self.m_xpos = dict[self.m_timepos].m_keysignature
     def get_width(self):
         #FIXME this value depends on the fontsize, and on what kind of
         #accidental we are drawing. A sharp is wider that a flat.
-        return len(self.m_accidentals) * 10 + 10
+        return len(self.m_accidentals) * 10 + len(self.m_old_accidentals) * 6 + 8
     def engrave(self, widget, gc, staff_yoffset):
         x = 0
-        for acc in self.m_accidentals:
+        dolist = []
+        # natural signs
+        for acc in self.m_old_accidentals:
+            if acc in self.m_accidentals:
+                continue
+            type = 0
             ypos = mpdutils.an_to_ylinepos(acc, self.m_clef)
-            if acc[-4:] == 'eses':
+            dolist.append((0, x, ypos))
+            #FIXME see FIXME msg in .get_width
+            x += 6
+        # accidentals
+        for acc in self.m_accidentals:
+            if acc.endswith('eses'):
                 type = -2
-            elif acc[-2:] == 'es':
+            elif acc.endswith('es'):
                 type = -1
-            elif acc[-4:] == 'isis':
+            elif acc.endswith('isis'):
                 type = 2
             else:
                 type = 1
+            ypos = mpdutils.an_to_ylinepos(acc, self.m_clef)
+            dolist.append((type, x, ypos))
+            #FIXME see FIXME msg in .get_width
+            x += 10
+        for type, x, ypos in dolist:
             im = gtk.Image()
             im.set_from_file(fetadir+'/feta%i-accidentals-%i.xpm' % (self.m_fontsize, type))
             im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
@@ -210,8 +227,6 @@ class KeySignatureEngraver(Engraver):
                      + dimentions[self.m_fontsize].linespacing*ypos/2
                      + accidental_y_offset[type], -1, -1,
                      gtk.gdk.RGB_DITHER_NONE, 0, 0)
-            #FIXME see FIXME msg in .get_width
-            x += 10
 
 
 class NoteheadEngraver(Engraver):
@@ -244,7 +259,7 @@ class NoteheadEngraver(Engraver):
         im.set_from_file(fetadir+'/feta20-dots-dot.xpm')
         for n in range(self.m_numdots):
             im.get_pixbuf().render_to_drawable(widget.window, gc, 0, 0,
-                        self.m_xpos+dim.xshift*(self.m_shift+1.5+n/2.0),
+                        int(self.m_xpos+dim.xshift*(self.m_shift+1.5+n/2.0)),
                         -3 + staff_yoffset + dim.linespacing*self.m_ypos/2,
                         -1, -1,
                      gtk.gdk.RGB_DITHER_NONE, 0, 0)
@@ -450,14 +465,14 @@ class StemEngraver(Engraver):
                 im.set_from_file(fetadir+'/feta%i-flags-%s.xpm' % (self.m_fontsize, self.get_flag(const.UP)))
                 im.get_pixbuf().render_to_drawable(widget.window, widget.get_style().black_gc, 0, 0,
                             self.m_xpos,
-                            staff_yoffset - self.m_stemlen + self.m_yposes[0] * dim.linespacing/2,
+                            int(staff_yoffset - self.m_stemlen + self.m_yposes[0] * dim.linespacing/2),
                             -1, -1, gtk.gdk.RGB_DITHER_NONE, 0, 0)
             else:
                 im = gtk.Image()
                 im.set_from_file(fetadir+'/feta%i-flags-%s.xpm' % (self.m_fontsize, self.get_flag(const.DOWN)))
                 im.get_pixbuf().render_to_drawable(widget.window, widget.get_style().black_gc, 0, 0,
                             self.m_xpos,
-                            staff_yoffset + 4 + self.m_yposes[-1] * dim.linespacing/2,
+                            int(staff_yoffset + 4 + self.m_yposes[-1] * dim.linespacing/2),
                             -1, -1, gtk.gdk.RGB_DITHER_NONE, 0, 0)
         # draw stem
         if self.m_mgroup.m_stemdir == const.DOWN:
@@ -544,7 +559,7 @@ class RestEngraver(Engraver):
         im.get_pixbuf().render_to_drawable(widget.window, widget.get_style().black_gc,
                     0, 0,
                     self.m_xpos,
-                    staff_yoffset - my + dim.linespacing*self.m_ypos/2 - 4,
+                    int(staff_yoffset - my + dim.linespacing*self.m_ypos/2 - 4),
                     -1, -1,
                     gtk.gdk.RGB_DITHER_NONE, 0, 0)
     def get_pixmap_id(self):
