@@ -141,7 +141,7 @@ class MusicalPitch:
         r.m_accidental_i = self.m_accidental_i
         return r
     def new_from_notename(n):
-        assert type(n) == type("")
+        assert type(n) in types.StringTypes
         r = MusicalPitch()
         r.set_from_notename(n)
         return r
@@ -195,14 +195,14 @@ class MusicalPitch:
             elif notename[-1] == ",":
                 self.m_octave_i = self.m_octave_i - 1
             notename = notename[:-1]
-        if notename[:2] == 'es':
+        if notename.startswith('es'):
             notename = 'ees' + notename[2:]
-        if notename[:2] == 'as':
+        if notename.startswith('as'):
             notename = 'aes' + notename[2:]
-        while notename[-2:] == 'es':
+        while notename.endswith('es'):
             self.m_accidental_i = self.m_accidental_i -1
             notename = notename[:-2]
-        while notename[-2:] == 'is':
+        while notename.endswith('is'):
             self.m_accidental_i = self.m_accidental_i + 1
             notename = notename[:-2]
         try:
@@ -214,9 +214,9 @@ class MusicalPitch:
         lowest and highest can be an integer, string or a MusicalPitch instance
         """
         assert type(lowest) == type(highest)
-        if type(lowest) == types.StringType:
+        if type(lowest) in  types.StringTypes:
             lowest = MusicalPitch.new_from_notename(lowest).semitone_pitch()
-        if type(highest) == types.StringType:
+        if type(highest) in types.StringTypes:
             highest = MusicalPitch.new_from_notename(highest).semitone_pitch()
         self.set_from_int(random.randint(int(lowest), int(highest)))
         return self
