@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,9 +42,9 @@ class RestRequest(ItHasDuration):
 class Music(ItHasDuration):
     def __init__(self, notename, duration, dots):
         ItHasDuration.__init__(self, duration, dots)
-        self.m_pitch = MusicalPitch(notename)
+        self.m_pitch = MusicalPitch.new_from_notename(notename)
     def __str__(self):
-        return "(Music:%s, %s)" % (self.m_pitch.str(), self.m_duration)
+        return "(Music:%s, %s)" % (self.m_pitch.get_octave_notename(), self.m_duration)
     def transpose(self, P):
         self.m_pitch.transpose_by_musicalpitch(P)
 
