@@ -82,7 +82,8 @@ class MelodicIntervalTeacher(Teacher):
         self.m_tonika = None
         self.m_question = []
     def new_question(self, L, H):
-        assert type(L) == type(H) == types.StringType
+        assert type(L) in types.StringTypes
+        assert type(H) in types.StringTypes
         if self.get_list('ask_for_intervals_0') == []:
             return self.ERR_CONFIGURE
         L, H = utils.adjust_low_high_to_irange(L, H,
