@@ -233,6 +233,10 @@ def dump(dict, fn):
     f.close()
 
 def drop_user_config():
+    """
+    Reread the config data, but only read the systems defaults, not
+    .solfegerc in the users $HOME.
+    """
     global data, _watches, _watch_counter
     data = {}
     _watches = {}
