@@ -27,7 +27,7 @@ except:
     pygtk.require('2.0')
 
 
-import re, string, types, os
+import re, string, types, os, sys
 import const
 import random
 import mpd, cfg, utils
@@ -80,6 +80,7 @@ predef = {'dictation': 'dictation',
 
 lessonfile_functions = {
         '_': _,
+        '_i': _i,
 }
 
 class _Header:
@@ -103,10 +104,8 @@ class LessonfileCommon:
         parser.parse_file(filename)
         self._idx = 0
         self.m_transpose = mpd.MusicalPitch.new_from_notename("c'")
-        H = filter(lambda e: e['blocktype'] == 'header', parser.m_blocks)
-        self.header = _Header(H[0])
-        self.m_questions = filter(lambda e: e['blocktype'] == 'question',
-                                  parser.m_blocks)
+        self.header = _Header(parser.header)
+        self.m_questions = parser.questions
         if utils.compare_version_strings(self.header.version, "1.1.3") == -1:
             def convert(q):
                 for n in ('music', 'clue_music'):
@@ -278,8 +277,6 @@ class SingChordLessonfile(LessonfileCommon):
             music = string.replace(music, r"\addvoice",
                       r"\addvoice\transpose %s" % self.m_transpose.get_octave_notename())
         return music
-    def handle_random_transpose_is_yes(self):
-        self.header.random_transpose = ['accidentals', -5, 5]
 
 class IdByNameLessonfile(LessonfileCommon):
     def __init__(self, filename):
@@ -291,8 +288,7 @@ class IdByNameLessonfile(LessonfileCommon):
         self.m_names_in_file_order = []
         for question in self.m_questions:
             if not question.has_key('name'):
-                print "discarding question in the lessonfile '%s'" % filename
-                print "because of missing 'name' variable"
+                print >> sys.stderr, "IdByNameLessonfile.parse_file: discarding question in the lessonfile '%s'\nbecause of missing 'name' variable" % filename
                 continue
             if not self.m_names.has_key(question['name']):
                 self.m_names[question['name']] = \
@@ -303,11 +299,6 @@ class IdByNameLessonfile(LessonfileCommon):
         Return the untranslated name for the selected question.
         """
         return self.m_questions[self._idx]['name']
-    def get_name(self):
-        """
-        Return the translated name for the selected question.
-        """
-        return self.m_names[self.m_questions[self._idx]['name']]
 
 class ChordLessonfile(LessonfileCommon):
     def __init__(self, filename):
@@ -377,12 +368,14 @@ class LessonFileManager:
                     continue
                 try:
                     p = dataparser.Dataparser(predef, lessonfile_functions)
-                    p.parse_file(self.get_complete_filename(collection, filename), 1)
-                except dataparser.DataparserException:
+                    p.parse_file(self.get_complete_filename(collection, filename))
+                except dataparser.DataparserException, e:
                     self.m_invalid_files.append((collection, filename))
+                    print >> sys.stderr, "Discarding lessonfile '%s/%s' because the file contain errors:" % (collection, filename)
+                    print >> sys.stderr, str(e)
                     continue
-                c = p.m_blocks[0]['content']
-                if type(c) == types.StringType:
+                c = p.header['content']
+                if type(c) in types.StringTypes:
                     c = [c]
                 for u in c:
                     if not self.m_db[collection].has_key(u):
@@ -394,11 +387,16 @@ class LessonFileManager:
         self.m_htmldoc = """<html><body><p>This list is updated each time
                 you start Solfege. As you can see, Solfege need more
                 lessonfiles. <a href="lessonfiles.html">Write one!</a>"""
+        if self.m_invalid_files:
+            self.m_htmldoc += "<h1>Files we failed to parse</h1><ul>"
+            for collection, filename in self.m_invalid_files:
+                self.m_htmldoc += "<li>%(filename)s in collection %(collection)s</li>" % {'collection': collection, 'filename': filename}
+            self.m_htmldoc += "</ul>"
         for collection in self.m_db.keys():
             self.m_htmldoc = self.m_htmldoc + "<h1>collection: %s</h1>" % collection
             for use in self.m_db[collection].keys():
-                self.m_htmldoc = self.m_htmldoc + "<h2>%s</h2>" % use
-                for file in self.m_db[collection][use]:
+                self.m_htmldoc = self.m_htmldoc + "<h2>%s</h2><p>" % use
+                for fn in self.m_db[collection][use]:
                     P = {const.USE_CHORD: 'chord',
                          const.USE_CHORD_VOICING: 'chord-voicing',
                          const.USE_HARMONY: 'harmonic-progression-dictation',
@@ -406,12 +404,8 @@ class LessonFileManager:
                          const.USE_DICTATION: 'dictation',
                          const.USE_SING_CHORD: 'sing-chord'}[use]
                     self.m_htmldoc = self.m_htmldoc + \
-                      "<a href=\"solfege:practise/%s/%s/%s\">%s</a> " % (P, collection, file, file)
-        if self.m_invalid_files:
-            self.m_htmldoc += "<h1>Files we failed to parse</h1><ul>"
-            for collection, filename in self.m_invalid_files:
-                self.m_htmldoc += "<li>%(collection)s in %(filename)s</li>" % {'collection': collection, 'filename': filename}
-            self.m_htmldoc += "</ul>"
+                      "<a href=\"solfege:practise/%s/%s/%s\">%s</a>\n" % (P, collection, fn, fn)
+                self.m_htmldoc += "</p>"
         self.m_htmldoc = self.m_htmldoc + "</body></html>"
     def get_collections(self):
         """
@@ -434,6 +428,13 @@ class LessonFileManager:
 
 if __name__ == '__main__':
     import src, src.i18n
-    p = LessonfileCommon()
-    p._parse_file(sys.argv[1])
-    print p.header.random_transpose
+    import time
+    cfg.initialise(None, "default.config",
+        "~/.solfegerc1.9")
+    t1=time.clock()
+    for n in range(10):
+        #p = LessonfileCommon()
+        #p._parse_file(sys.argv[1])
+        p = LessonFileManager()
+    t2 = time.clock()
+    print t2-t1
