@@ -46,13 +46,13 @@ This is free software. It is covered by the GNU General Public License,
 and you are welcome to change it and/or distribute copies of it under
 certain conditions. Invoke as `solfege --warranty` for more information.
 
-Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen <tca@gnu.org>
+Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen <tca@gnu.org>
             """ % configureoutput.VERSION_STRING
             sys.exit()
 
         if '-w' in argv or '--warranty' in argv:
             print """GNU Solfege %s
-Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen <tca@gnu.org>
+Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen <tca@gnu.org>
 
     This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License version 2
@@ -156,21 +156,24 @@ gnome.ui.PAD_BIG=12
 if splash_win:
     splash_win.show_progress("importing application modules")
 
+import tracebackwindow
+# redirect error messages to a window that will popup if
+# something bad happens.
+
+sys.stderr = tracebackwindow.TracebackWindow()
+
 import const, string
 import app
 from htmlwidget import HtmlWidget
 import utils, history
 from configwindow import ConfigWindow
 import mpd, mpd.musicdisplayer, gu, cfg
-import tracebackwindow
-
-
-# redirect error messages to a window that will popup if
-# something bad happens.
-
-w = tracebackwindow.TracebackWindow()
-sys.stderr = w
+import soundcard
 
+import chord, chordvoicing, harmonicinterval, melodicinterval, \
+       singinterval, identifyscale, idbyname, dictation, twelvetone, \
+       idtone, compareintervals, singchord, rhythm, identifybpm, \
+       harmonicprogressiondictation
 
 class DocViewer(gtk.VBox):
     def __init__(self, activate_cb):
@@ -271,11 +274,12 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
                 trans = ""
             self.g_about_window = gnome.ui.About("GNU Solfege",
                   configureoutput.VERSION_STRING,
-                  "Copyright (C) 2000, 2001, 2002 Tom Cato Amundsen",
+                  "Copyright (C) 2000, 2001, 2002, 2003 Tom Cato Amundsen",
                   _("GPL'ed eartraining.") + "\n" + "http://www.solfege.org",
                   ["Tom Cato Amundsen",
                   'Oliver Vollmer %s' % _("(toolbar icons)"),
                   'Giovanni Chierico %s' % _("(some lessonfiles)"),
+                  'Mickael Becker %s' % _("(some lessonfiles)"),
                   'Joe Lee %s' % _("(sound code for the MS Windows port)"),
                   ],
                   ["Tom Cato Amundsen"],
@@ -295,6 +299,8 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         if not self.get_bool("gui/toolbar_visible"):
             self.get_dock_item_by_name('toolbar').hide()
         self.box_dict['docviewer'].grab_focus()
+        if not soundcard.synth:
+            self.display_error_message(_("Sound support is not configured yet. Select 'Preferences window' from the File menu, then select the page named 'Sound setup'."))
     def activate_exercise(self, href):
         protocol, action, exname, anchor, collection, lessonfile, config = \
            utils.parse_url(href)
@@ -355,8 +361,7 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
     def initialise_exercise(self, teacher):
         assert not self.box_dict.has_key(teacher.m_exname)
         n = utils.exercise_name_to_module_name(teacher.m_exname)
-        exec("import %s" % n)
-        self.box_dict[teacher.m_exname] = locals()[n].Gui(teacher, self)
+        self.box_dict[teacher.m_exname] = globals()[n].Gui(teacher, self)
         self.main_box.pack_start(self.box_dict[teacher.m_exname])
     def on_key_press_event(self, widget, event):
         if event.type == gtk.gdk.KEY_PRESS:
@@ -596,6 +601,7 @@ locale.setlocale(locale.LC_NUMERIC, "C")
 
 if splash_win:
     splash_win.show_progress("creating MainWin")
+
 w = MainWin(args)
 w.show()
 w.post_constructor()
