@@ -1,5 +1,5 @@
 # GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,85 +16,80 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import utils, gtk, gu, widgets
-from i18n import _
 import gnome
 import os
 
 
-class MatrixTable(gtk.GtkVBox):
+class MatrixTable(gtk.VBox):
     def __init__(self, label_style, heading, keys, key_to_pretty_name, dict):
-        gtk.GtkVBox.__init__(self)
-        label = gtk.GtkLabel(heading)
+        gtk.VBox.__init__(self)
+        label = gtk.Label(heading)
         label.set_name("Heading2")
         label.set_alignment(0.0, 0.0)
         self.pack_start(label, gtk.FALSE)
         hbox = gu.bHBox(self, gtk.FALSE)
-        frame = gtk.GtkFrame()
+        frame = gtk.Frame()
         hbox.pack_start(frame, gtk.FALSE)
-        t = gtk.GtkTable()
+        t = gtk.Table()
         frame.add(t)
         for x in range(len(keys)):
-            t.attach(gtk.GtkVSeparator(), x*2+1, x*2+2, 0, len(keys)*2)
+            t.attach(gtk.VSeparator(), x*2+1, x*2+2, 0, len(keys)*2)
         for x in range(len(keys)-1):
-            t.attach(gtk.GtkHSeparator(), 0, len(keys)*2+1, x*2+1, x*2+2)
+            t.attach(gtk.HSeparator(), 0, len(keys)*2+1, x*2+1, x*2+2)
         for y in range(len(keys)):
             if label_style == 'progression':
                 l = widgets.HarmonicProgressionLabel(
                                      key_to_pretty_name(keys[y]), 'left')
             else:
-                l = gtk.GtkLabel(_(key_to_pretty_name(keys[y])))
+                l = gtk.Label(key_to_pretty_name(keys[y]))
                 l.set_alignment(0.0, 0.5)
-            t.attach(l, 0, 1, y*2, y*2+1, xpadding=gnome.uiconsts.PAD)
+            t.attach(l, 0, 1, y*2, y*2+1, xpadding=gnome.ui.PAD)
             for x in range(len(keys)):
                 if dict.has_key(keys[x]) and dict[keys[x]].has_key(keys[y]):
                     s = str(dict[keys[x]][keys[y]])
                 else:
                     s = "0"
-                l = gtk.GtkLabel(s)
+                l = gtk.Label(s)
                 if x == y:
                     l.set_name('StatisticsViewerDiagonal')
-                t.attach(l, x*2+2, x*2+3, y*2, y*2+1, xpadding=gnome.uiconsts.PAD)
+                t.attach(l, x*2+2, x*2+3, y*2, y*2+1, xpadding=gnome.ui.PAD)
         self.show_all()
 
 
-class PercentagesTable(gtk.GtkFrame):
-    def __init__(self):
-        gtk.GtkFrame.__init__(self)
-        table = gtk.GtkTable()
+class PercentagesTable(gtk.Frame):
+    def __init__(self, statistics):
+        gtk.Frame.__init__(self)
+        table = gtk.Table()
         self.add(table)
 
         self.boxdict = {}
         for k, l, x in (('session', _("Session"), 2), ('today', _("Today"), 5),
                      ('last7', _("Last 7 days"), 8), ('total', _("Total"), 11)):
-            table.attach(gtk.GtkLabel(l), x, x+2, 0, 1)
-            b = gtk.GtkVBox()
+            table.attach(gtk.Label(l), x, x+2, 0, 1)
+            b = gtk.VBox()
             table.attach(b, x, x+1, 4, 5)
             self.boxdict[k+'percent'] = b
-            b = gtk.GtkVBox()
+            b = gtk.VBox()
             table.attach(b, x+1, x+2, 4, 5)
             self.boxdict[k+'count'] = b
         for x in (2, 5, 8, 11):
-            table.attach(gtk.GtkLabel(_("Percent")), x, x+1, 2, 3)
-            table.attach(gtk.GtkLabel(_("Count")), x+1, x+2, 2, 3)
-        table.attach(gtk.GtkHSeparator(), 0, 13, 1, 2)
-        table.attach(gtk.GtkHSeparator(), 0, 13, 3, 4)
-        table.attach(gtk.GtkVSeparator(), 1, 2, 0, 6)
-        table.attach(gtk.GtkVSeparator(), 4, 5, 0, 6)
-        table.attach(gtk.GtkVSeparator(), 7, 8, 0, 6)
-        table.attach(gtk.GtkVSeparator(), 10, 11, 0, 6)
-        self.boxdict['keys'] = key_box = gtk.GtkVBox()
+            table.attach(gtk.Label(_("Percent")), x, x+1, 2, 3)
+            table.attach(gtk.Label(_("Count")), x+1, x+2, 2, 3)
+        table.attach(gtk.HSeparator(), 0, 13, 1, 2)
+        table.attach(gtk.HSeparator(), 0, 13, 3, 4)
+        table.attach(gtk.VSeparator(), 1, 2, 0, 6)
+        table.attach(gtk.VSeparator(), 4, 5, 0, 6)
+        table.attach(gtk.VSeparator(), 7, 8, 0, 6)
+        table.attach(gtk.VSeparator(), 10, 11, 0, 6)
+        self.boxdict['keys'] = key_box = gtk.VBox()
         table.attach(key_box, 0, 1, 4, 5)
         for box in self.boxdict.values():
-            box.set_border_width(gnome.uiconsts.PAD_SMALL)
+            box.set_border_width(gnome.ui.PAD_SMALL)
+        self.update(statistics)
         self.show_all()
-    def clear(self):
-        for box in self.boxdict.values():
-            for o in box.children():
-                o.destroy()
     def update(self, statistics):
-        self.clear()
         for box in self.boxdict.values():
-            for o in box.children():
+            for o in box.get_children():
                 o.destroy()
         for k in statistics.get_keys():
             # FIXME  Has this bug been fixed? This should NEVER happen.
@@ -106,7 +101,7 @@ class PercentagesTable(gtk.GtkFrame):
             if statistics.get_label_style() == 'progression':
                 l = widgets.HarmonicProgressionLabel(statistics.key_to_pretty_name(k), 'left')
             else:
-                l = gtk.GtkLabel(_(statistics.key_to_pretty_name(k)))
+                l = gtk.Label(statistics.key_to_pretty_name(k))
                 l.set_alignment(0.0, 0.5)
             self.boxdict['keys'].pack_start(l)
             for sk, st in (('session', statistics.m_session_stat),
@@ -115,51 +110,66 @@ class PercentagesTable(gtk.GtkFrame):
                        ('total', statistics.m_total_stat)):
 
                 self.boxdict[sk+'percent'].pack_start(
-                    gtk.GtkLabel("%.0f" % statistics.get_percentage_correct(st, k)))
+                    gtk.Label("%.0f" % statistics.get_percentage_correct(st, k)))
                 self.boxdict[sk+'count'].pack_start(
-                    gtk.GtkLabel(str(statistics.get_num_guess(st, k))))
+                    gtk.Label(str(statistics.get_num_guess(st, k))))
         self.show_all()
 
-class NewAbstractStatisticsViewer(gtk.GtkScrolledWindow):
+class NewAbstractStatisticsViewer(gtk.ScrolledWindow):
     def __init__(self, statistics, heading):
-        gtk.GtkScrolledWindow.__init__(self)
+        gtk.ScrolledWindow.__init__(self)
         self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.vbox = gtk.GtkVBox()
-        self.vbox.set_spacing(gnome.uiconsts.PAD)
-        self.vbox.set_border_width(gnome.uiconsts.PAD)
+        self.vbox = gtk.VBox()
+        self.vbox.set_spacing(gnome.ui.PAD)
+        self.vbox.set_border_width(gnome.ui.PAD)
         self.add_with_viewport(self.vbox)
-        label = gtk.GtkLabel(heading)
+        label = gtk.Label(heading)
         label.set_alignment(0.0, 0.5)
         label.set_name("Heading1")
         self.vbox.pack_start(label, gtk.FALSE)
         self.m_statistics = statistics
-        self.g_p = PercentagesTable()
-        self.g_p.show_all()
-        hbox = gtk.GtkHBox()
-        self.vbox.pack_start(hbox, gtk.FALSE)
-        hbox.pack_start(self.g_p, gtk.FALSE)
+        self.g_tables = gtk.VBox()
+        self.g_tables.show()
+        self.vbox.pack_start(self.g_tables)
         self.show_all()
 
-class SingIntervallStatisticsViewer(NewAbstractStatisticsViewer):
+class SingIntervalStatisticsViewer(NewAbstractStatisticsViewer):
     def __init__(self, statistics, heading):
         NewAbstractStatisticsViewer.__init__(self, statistics, heading)
-        self.clear = self.g_p.clear
+        #self.clear = self.g_p.clear
+    def clear(self):
+        #UGH why cant we just destroy the children of g_tables??!!
+        #for c in self.g_tables.children():
+        #    c.destroy()
+        self.g_tables.destroy()
+        self.g_tables = gtk.VBox()
+        self.g_tables.show()
+        self.vbox.pack_start(self.g_tables)
     def update(self):
-        self.g_p.update(self.m_statistics)
+        self.clear()
+        self.g_p = PercentagesTable(self.m_statistics)
+        self.g_p.show_all()
+        self.g_tables.pack_start(self.g_p, gtk.FALSE, gtk.FALSE)
 
 
-class GtkStatisticsViewer(NewAbstractStatisticsViewer):
+
+class StatisticsViewer(NewAbstractStatisticsViewer):
     def __init__(self, statistics, heading):
         NewAbstractStatisticsViewer.__init__(self, statistics, heading)
         self.matrix_dict = {}
     def clear(self):
-        self.g_p.clear()
-        for k in ('session', 'today', 'last7', 'total'):
-            if self.matrix_dict.has_key(k):
-                self.matrix_dict[k].destroy()
+        #UGH why cant we just destroy the children of g_tables??!!
+        #for c in self.g_tables.children():
+        #    c.destroy()
+        self.g_tables.destroy()
+        self.g_tables = gtk.VBox()
+        self.g_tables.show()
+        self.vbox.pack_start(self.g_tables)
     def update(self):
         self.clear()
-        self.g_p.update(self.m_statistics)
+        self.g_p = PercentagesTable(self.m_statistics)
+        self.g_p.show_all()
+        self.g_tables.pack_start(self.g_p, gtk.FALSE)
         self.matrix_dict['session'] = MatrixTable(
                                            self.m_statistics.get_label_style(), 
                                            _("Session"),
@@ -186,8 +196,8 @@ class GtkStatisticsViewer(NewAbstractStatisticsViewer):
                                            self.m_statistics.m_total_stat)
 
         for k in ('session', 'today', 'last7', 'total'):
-            self.vbox.pack_start(self.matrix_dict[k], gtk.FALSE)
+            self.g_tables.pack_start(self.matrix_dict[k], gtk.FALSE)
     #def transform_key(self, k):
-    #    return utils.int_to_intervallname(k, 0, 0)
+    #    return utils.int_to_intervalname(k, 0, 0)
 
-StatisticsViewer = GtkStatisticsViewer
+StatisticsViewer = StatisticsViewer
